@@ -1,7 +1,7 @@
 /*
  * bg.h
  *
- * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
+ * Copyright (C) 2019 Peter Belkner <info@pbelkner.de>
  * Nanos gigantum humeris insidentes #TeamWhite
  *
  * This program is free software; you can redistribute it and/or
@@ -25,6 +25,21 @@
 #include <config.h>
 #endif // ]
 #include <stdio.h>
+
+#define BG_PARAM_QUIET
+#if defined (BG_PARAM_QUIET) // [
+#if defined (HAVE_PTHREAD) || defined (_WIN32) // [
+#define BG_PARAM_THREADS
+#endif // ]
+#if defined (BG_PARAM_THREADS) // [
+// don't define!
+//#define BG_VISITOR_NOTIFY_PARENT
+#endif // ]
+#if defined (HAVE_PTHREAD) // [
+#include <pthread.h>
+#endif // ]
+#endif // ]
+
 #include <pbutil_priv.h>
 #include <ff.h>
 #include <lib1770.h>
@@ -35,13 +50,37 @@ extern "C" {
 #endif // ]
 
 ///////////////////////////////////////////////////////////////////////////////
-//#define BG_LIST
-#define BG_CHANNEL_LFE 3
+#if ! defined (_WIN32) // [
+//#define BG_UTF8_ITER
+#if defined (BG_UTF8_ITER) // [
+//#define BG_UTF8_ITER_WCS
+#define BG_UTF8_ITER_MASK
+#else // ] [
+#define BG_CHAR_NEXTA
+#endif // ]
+#endif // ]
+
+#define BG_PARAM_SKIP_SCAN
+#define BG_PARAM_REFERENCE
+//#define BG_PARAM_NODE_VMT
+#define BG_PARAM_NODE_VMT
+#if defined (BG_PARAM_NODE_VMT) // [
+#define BG_PARAM_SCRIPT
+#if defined (BG_PARAM_SCRIPT) // [
+#define BG_PARAM_SHELL
+#endif // ]
+#endif // ]
+
 #define BG_TEMP_PREFIX FFL(".")
 #define BG_CLOCK
+#define BG_TRACK_ID
 //#define BG_PARAM_SLEEP
 //#define BG_BWF_TAGS
 //#define BG_TREE_CREATE_CHILD_WARNING
+//#define BG_PARAM_LFE
+#if defined (BG_PARAM_LFE) // [
+#define BG_CHANNEL_LFE 3
+#endif // ]
 #if defined (_WIN32) // [
 #define BG_WIN32_TARGET_UTF8
 // WARNING:  define BG_WIN32_CREATE_LOCALE at your own risk: at the run-time
@@ -76,6 +115,26 @@ typedef enum bg_flags_norm bg_flags_norm_t;
 typedef const struct bg_print_vmt bg_print_vmt_t;
 typedef const struct bg_print_conf bg_print_conf_t;
 typedef const struct bg_param_unit bg_param_unit_t;
+#if defined (BG_UTF8_ITER) // [
+typedef const struct bg_utf8_iter_vmt bg_utf8_iter_vmt_t;
+#if defined (BG_UTF8_ITER_MASK) // [
+typedef enum bg_utf8_iter_mask bg_utf8_iter_mask_t;
+#endif // ]
+typedef struct bg_utf8_iter bg_utf8_iter_t;
+#endif // ]
+#if defined (BG_PARAM_THREADS) // [
+typedef struct bg_sync bg_sync_t;
+typedef struct bg_threads_helper bg_threads_helper_t;
+typedef int (bg_dispatch_t)(bg_tree_t *tree, bg_visitor_t *vis);
+typedef enum bg_param_request_tag bg_param_request_tag_t;
+typedef struct bg_param_request bg_param_request_t;
+#if defined (BG_PARAM_NODE_VMT) // [
+typedef struct bg_param_node_vmt bg_param_node_vmt_t;
+#endif // ]
+typedef struct bg_param_node bg_param_node_t;
+typedef struct bg_param_list bg_param_list_t;
+typedef struct bg_param_threads bg_param_threads_t;
+#endif // ]
 typedef struct bg_param bg_param_t;
 typedef struct bg_param_argv bg_param_argv_t;
 
@@ -105,13 +164,14 @@ struct bg_annotation_vmt {
 };
 
 struct bg_tree_vmt {
-#if defined (PBU_DEBUG) // [
   const ffchar_t *id;
-#endif // ]
   const bg_tree_type_t type;
   void (*destroy)(bg_tree_t *tree);
   int (*accept)(bg_tree_t *tree, bg_visitor_t *vis);
   bg_annotation_vmt_t annotation;
+#if defined (BG_TRACK_ID) // [
+  void (*track_id)(bg_tree_t *tree, int *id);
+#endif // ]
 };
 
 ////////
@@ -123,8 +183,18 @@ void bg_root_annotation_destroy(bg_tree_t *tree);
 ////////
 struct bg_album {
   // counts BG_TREE_TYPE_FILEs and BG_TREE_TYPE_TRACKs.
-  unsigned nchildren;
+	struct {
+  	unsigned max,cur;
+	} nchildren;
+
   unsigned nleafs;
+
+#if defined (BG_TRACK_ID) // [
+  struct {
+    int id;
+  } track;
+#endif // ]
+
   bg_tree_t *first;
   bg_tree_t *last;
 };
@@ -195,7 +265,11 @@ struct bg_track {
   } root;
 
   struct {
+#if defined (BG_TRACK_ID) // [
+    int id;
+#else // ] [
     unsigned long id;
+#endif // ]
   } album;
 
   struct {
@@ -239,7 +313,79 @@ int bg_tree_patha_create(bg_tree_patha_t *p, const wchar_t *path,
 void bg_tree_patha_destroy(bg_tree_patha_t *p);
 #endif // ]
 
+#if defined (BG_UTF8_ITER) // [
+///////////////////////////////////////////////////////////////////////////////
+struct bg_utf8_iter_vmt {
+	int (*valid)(bg_utf8_iter_t *i);
+	const uint8_t *(*next)(bg_utf8_iter_t *i);
+	void (*apply)(bg_utf8_iter_t *i);
+	void (*flush)(bg_utf8_iter_t *i);
+};
+
+#if defined (BG_UTF8_ITER_MASK) // [
+enum bg_utf8_iter_mask {
+  // cf. e.g. "https://en.wikipedia.org/wiki/UTF-8#Description".
+  BG_UTF8_ITER_MASK_ONE_BYTE=~(1u<<7u),
+  BG_UTF8_ITER_MASK_TWO_BYTES=~(1u<<5u),
+  BG_UTF8_ITER_MASK_THREE_BYTES=~(1u<<4u),
+  BG_UTF8_ITER_MASK_FOUR_BYTES=~(1u<<3u),
+	BG_UTF8_ITER_MBS_SIZE=4,
+};
+#else // ] [
+#error not implemented yet
+#endif // ]
+
+struct bg_utf8_iter {
+	bg_utf8_iter_vmt_t *vmt;
+#if defined (BG_UTF8_ITER_MASK) // [
+	uint8_t mbs[BG_UTF8_ITER_MBS_SIZE];
+#endif // ]
+	const uint8_t *rp;
+	uint8_t *wp;
+	size_t *size;
+#if defined (BG_UTF8_ITER_WCS) // [
+	uint8_t wcs;
+#endif // ]
+};
+
+void bg_utf8_iter_first(bg_utf8_iter_t *i, const char *rp, char *wp,
+		size_t *size);
+#elif defined (BG_CHAR_NEXTA) // ] [
+const char *bg_char_nexta(const char *str);
+#elif defined (_WIN32) // ] [
+#define bg_char_nexta(str) CharNextA(str)
+#endif // ]
+
+#if defined (BG_PARAM_THREADS) // [
 ////////
+struct bg_sync {
+#if defined (HAVE_PTHREAD) // [
+  pthread_mutex_t mutex;
+  pthread_cond_t cond;
+#elif defined (_WIN32) // ] [
+  CRITICAL_SECTION cs;
+  HANDLE hEvent;
+#endif // ]
+};
+
+int bg_sync_create(bg_sync_t *sync);
+void bg_sync_destroy(bg_sync_t *sync);
+
+void bg_sync_lock(bg_sync_t *sync);
+void bg_sync_unlock(bg_sync_t *sync);
+void bg_sync_signal(bg_sync_t *sync);
+void bg_sync_wait(bg_sync_t *sync);
+
+////////
+struct bg_threads_helper {
+  unsigned nchildren;
+  bg_sync_t sync;
+};
+
+int bg_threads_helper_create(bg_threads_helper_t *helper);
+void bg_threads_helper_destroy(bg_threads_helper_t *helper);
+#endif // ]
+
 struct bg_tree {
   bg_tree_vmt_t *vmt;
   bg_param_t *param;
@@ -272,6 +418,10 @@ struct bg_tree {
     double truepeak;
   } stats;
 
+#if defined (BG_PARAM_THREADS) // [
+  bg_threads_helper_t helper;
+#endif // ]
+
   union {
     bg_album_t album;
     bg_track_t track;
@@ -285,12 +435,16 @@ int bg_tree_stats_create(bg_tree_t *tree);
 void bg_tree_stats_destroy(bg_tree_t *tree);
 
 // narrow character representation of the basename.
-const char *bg_tree_in_basanamen(bg_tree_t *tree);
-const char *bg_tree_out_basanamen(bg_tree_t *tree);
+const char *bg_tree_in_basename(bg_tree_t *tree);
+const char *bg_tree_out_basename(bg_tree_t *tree);
 #if defined (_WIN32) // [
 // wide character representation of the basename.
-const wchar_t *bg_tree_in_basanamew(bg_tree_t *tree);
-const wchar_t *bg_tree_out_basanamew(bg_tree_t *tree);
+const wchar_t *bg_tree_in_basenamew(bg_tree_t *tree);
+const wchar_t *bg_tree_out_basenamew(bg_tree_t *tree);
+#endif // ]
+
+#if defined (BG_PARAM_SCRIPT) // [
+int bg_process_tree_run_script(bg_tree_t *tree);
 #endif // ]
 
 int bg_leaf_create(bg_tree_t **tree, bg_param_t *param, bg_tree_t *parent,
@@ -300,6 +454,36 @@ double *bg_tree_samplepeak(bg_tree_t *tree);
 double *bg_tree_truepeak(bg_tree_t *tree);
 int bg_tree_merge(bg_tree_t *lhs, const bg_tree_t *rhs);
 
+///////////////////////////////////////////////////////////////////////////////
+// from "bg_print_conf.c" [
+double bg_print_conf_norm(bg_tree_t *tree);
+double bg_print_conf_momentary_mean(bg_tree_t *tree);
+double bg_print_conf_momentary_mean_relative(bg_tree_t *tree);
+double bg_print_conf_momentary_maximum(bg_tree_t *tree);
+double bg_print_conf_momentary_maximum_relative(bg_tree_t *tree);
+double bg_print_conf_momentary_range(bg_tree_t *tree);
+double bg_print_conf_shortterm_mean(bg_tree_t *tree);
+double bg_print_conf_shortterm_mean_relative(bg_tree_t *tree);
+double bg_print_conf_shortterm_maximum(bg_tree_t *tree);
+double bg_print_conf_shortterm_maximum_relative(bg_tree_t *tree);
+double bg_print_conf_shortterm_range(bg_tree_t *tree);
+double bg_print_conf_samplepeak_absolute(bg_tree_t *tree);
+double bg_print_conf_samplepeak_relative(bg_tree_t *tree);
+double bg_print_conf_truepeak_absolute(bg_tree_t *tree);
+double bg_print_conf_truepeak_relative(bg_tree_t *tree);
+
+const char *bg_print_conf_unit_lum(bg_tree_t *tree);
+const char *bg_print_conf_unit_lram(bg_tree_t *tree);
+const char *bg_print_conf_unit_spm(bg_tree_t *tree);
+const char *bg_print_conf_unit_tpm(bg_tree_t *tree);
+#if defined (_WIN32) // [
+const wchar_t *bg_print_conf_unit_luw(bg_tree_t *tree);
+const wchar_t *bg_print_conf_unit_lraw(bg_tree_t *tree);
+const wchar_t *bg_print_conf_unit_spw(bg_tree_t *tree);
+const wchar_t *bg_print_conf_unit_tpw(bg_tree_t *tree);
+#endif // ]
+// ]
+
 ///////////////////////////////////////////////////////////////////////////////
 // several phases are implemented by means of the visitor pattern (cf. e.g.
 // https://en.wikipedia.org/wiki/Visitor_pattern.) we've got the following
@@ -319,10 +503,16 @@ struct bg_visitor_vmt {
 };
 
 int bg_analyzer_create(bg_visitor_t *vis);
-int bg_muxer_create(bg_visitor_t *vis);
-
 int bg_analyzer_album_prefix(bg_visitor_t *vis, bg_tree_t *tree);
 int bg_analyzer_album_suffix(bg_visitor_t *vis, bg_tree_t *tree);
+#if 0 && defined (BG_PARAM_THREADS) // [
+int bg_analyzer_track(bg_visitor_t *vis, bg_tree_t *tree);
+#endif // ]
+
+int bg_muxer_create(bg_visitor_t *vis);
+#if 0 && defined (BG_PARAM_THREADS) // [
+int bg_muxer_track(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree);
+#endif // ]
 
 ////////
 struct bg_visitor {
@@ -461,6 +651,115 @@ struct bg_param_argv {
   unsigned lift;
 };
 
+#if defined (BG_PARAM_THREADS) // [
+////////
+enum bg_param_request_tag {
+  bg_param_request_tag_null,
+  bg_param_request_tag_kill,
+  bg_param_request_tag_visitor_run,
+};
+
+////////
+struct bg_param_request {
+  bg_param_request_tag_t tag;
+#if defined (BG_PARAM_SCRIPT) // [
+  ffchar_t *script;
+#endif // ]
+  bg_tree_t *tree;
+  bg_visitor_t *visitor;
+  bg_dispatch_t *dispatch;
+};
+
+void bg_param_request_clear(bg_param_request_t *request);
+
+////////
+#if defined (BG_PARAM_NODE_VMT) // [
+struct bg_param_node_vmt {
+  void (*run)(bg_param_node_t *node, bg_param_request_t *request);
+};
+#endif // ]
+
+struct bg_param_node {
+#if defined (BG_PARAM_NODE_VMT) // [
+  bg_param_node_vmt_t *vmt;
+#endif // ]
+  bg_param_node_t *prev,*next;
+  bg_param_threads_t *threads;
+  bg_param_request_t request;
+  bg_sync_t sync;
+#if defined (_WIN32) // [
+  HANDLE hThread;
+#else // ] [
+  pthread_t thread;
+#endif // ]
+};
+
+#if defined (BG_PARAM_NODE_VMT) // [
+#define bg_node_run(node,request) \
+  ((node)->vmt->run(node,request))
+#endif // ]
+
+int bg_param_node_create(bg_param_node_t *node, bg_param_threads_t *threads);
+#if 0 // [
+void bg_param_node_destroy(bg_param_node_t *node, int destroy);
+#else // ] [
+void bg_param_node_destroy(bg_param_node_t *node);
+#endif // ]
+
+#if defined (BG_PARAM_SCRIPT) // [
+void bg_param_node_request(bg_param_node_t *node, bg_param_request_tag_t tag,
+    ffchar_t *script, bg_tree_t *tree, bg_visitor_t *visitor,
+    bg_dispatch_t *dispatch);
+#else // ] [
+void bg_param_node_request(bg_param_node_t *node, bg_param_request_tag_t tag,
+    bg_tree_t *tree, bg_visitor_t *visitor, bg_dispatch_t *dispatch);
+#endif // ]
+
+////////
+struct bg_param_list {
+  int count;
+  bg_param_node_t *head,*tail;
+};
+
+int bg_param_list_create(bg_param_list_t *list, int n, bg_param_node_t *nodes,
+    bg_param_threads_t *threads);
+#if 0 // [
+void bg_param_list_destroy(bg_param_list_t *list, int destroy);
+#else
+void bg_param_list_destroy(bg_param_list_t *list);
+#endif // ]
+
+bg_param_node_t *bg_param_list_unlink(bg_param_list_t *list,
+    bg_param_node_t *node);
+bg_param_node_t *bg_param_list_pop(bg_param_list_t *list);
+bg_param_node_t *bg_param_list_push(bg_param_list_t *list,
+    bg_param_node_t *node);
+
+////////
+struct bg_param_threads {
+  bg_sync_t sync;
+  bg_param_node_t *nodes;
+
+  struct {
+    bg_param_list_t free;
+    bg_param_list_t active;
+  } list;
+};
+
+int bg_param_threads_create(bg_param_threads_t *threads, int n);
+void bg_param_threads_destroy(bg_param_threads_t *threads);
+
+#if defined (BG_PARAM_SCRIPT) // [
+void bg_param_threads_visitor_run(bg_param_threads_t *threads,
+    ffchar_t *script, bg_visitor_t *visitor, bg_tree_t *tree,
+    bg_dispatch_t *dispatch);
+#else // ] [
+void bg_param_threads_visitor_run(bg_param_threads_t *threads,
+    bg_visitor_t *visitor, bg_tree_t *tree, bg_dispatch_t *dispatch);
+#endif // ]
+void bg_param_threads_drain(bg_param_threads_t *threads);
+#endif // ]
+
 ////////
 struct bg_param {
   struct {
@@ -475,6 +774,23 @@ struct bg_param {
   } count;
 
   int process;
+#if defined (BG_PARAM_SKIP_SCAN) // [
+  int skip_scan;
+#endif // ]
+#if defined (BG_PARAM_REFERENCE) // [
+  ffchar_t *reference;
+#endif // ]
+#if defined (BG_PARAM_SCRIPT) // [
+  ffchar_t *script;
+#endif // ]
+
+#if defined (BG_PARAM_SHELL) // [
+  struct {
+    ffchar_t *interpreter;
+    ffchar_t *parameter;
+  } shell;
+#endif // ]
+
   bg_pilot_t pilot;
   bg_visitor_t analyzer;
   bg_tree_t root,*tos;
@@ -512,6 +828,7 @@ struct bg_param {
 
   struct {
     FILE *f;
+    int bits;
   } result;
 
   struct {
@@ -544,7 +861,9 @@ struct bg_param {
 #if defined (BG_CLOCK) // [
   int time;
 #endif // ]
+#if defined (BG_PARAM_LFE) // [
   int lfe;
+#endif // ]
 #if defined (_WIN32) && defined (BG_WIN32_CREATE_LOCALE) // [
   _locale_t locale;
 #endif // ]
@@ -569,6 +888,13 @@ struct bg_param {
     int64_t duration;
   } interval;
 
+#if defined (BG_PARAM_QUIET) // [
+  int quiet;
+#endif // ]
+#if defined (BG_PARAM_THREADS) // [
+  int nthreads;
+  bg_param_threads_t threads;
+#endif // ]
 #if defined (BG_PARAM_SLEEP) // [
   int sleep;
 #endif // ]
