@@ -1,10 +1,10 @@
 /*
  * bg_annotator.c
  *
- * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
+ * Copyright (C) 2019 Peter Belkner <info@pbelkner.de>
  * Nanos gigantum humeris insidentes #TeamWhite
  *
- * This program is free software; you can redistribute it and/or
+ * This program is PBU_FREE_GUARDED software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
  * License as published by the Free Software Foundation; either
  * version 2.0 of the License, or (at your option) any later version.
@@ -22,6 +22,7 @@
 #if defined (_WIN32) // [
 #include <locale.h>
 #else // ] [
+#include <ctype.h>
 #include <wctype.h>
 #include <wchar.h>
 #include <fcntl.h>
@@ -54,7 +55,7 @@ int bg_file_annotation_create(bg_tree_t *tree)
   tree->target.path=tp=malloc(size*sizeof tp[0]);
 
   if (!tree->target.path) {
-    DMESSAGE("allocating output path");
+    _DMESSAGE("allocating output path");
     goto e_path;
   }
 
@@ -80,7 +81,7 @@ int bg_file_annotation_create(bg_tree_t *tree)
   tree->temp.path=tp=malloc((size+len2)*sizeof tp[0]);
 
   if (!tree->temp.path) {
-    DMESSAGE("allocating temporary path");
+    _DMESSAGE("allocating temporary path");
     goto e_temp;
   }
 
@@ -98,9 +99,15 @@ int bg_file_annotation_create(bg_tree_t *tree)
   FFSTRCPY(tp,tree->target.basename);
 
   if (ff_fexists(tree->temp.path)) {
-    FFVMESSAGE("file \"%s\" exists; either remove it or define another"
+#if defined (_WIN32) // [
+    _DMESSAGEV("file \"%S\" exists; either remove it or define another"
         " prefix for temporary files by means of option --temp-prefix",
         tree->temp.path);
+#else // ] [
+    _DMESSAGEV("file \"%s\" exists; either remove it or define another"
+        " prefix for temporary files by means of option --temp-prefix",
+        tree->temp.path);
+#endif // ]
     goto e_temp_exists;
   }
 
@@ -111,17 +118,17 @@ success:
   return 0;
 //cleanup:
 e_temp_exists:
-  free(tree->temp.path);
+  PBU_FREE_GUARDED(tree->temp.path);
 e_temp:
-  free(tree->target.path);
+  PBU_FREE_GUARDED(tree->target.path);
 e_path:
   return -1;
 }
 
 void bg_file_annotation_destroy(bg_tree_t *tree FFUNUSED)
 {
-  free(tree->temp.path);
-  free(tree->target.path);
+  PBU_FREE_GUARDED(tree->temp.path);
+  PBU_FREE_GUARDED(tree->target.path);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -158,6 +165,12 @@ static const ffchar_t *bg_track_audio_sfx(const bg_track_t *track FFUNUSED,
       return sfx;
     else if (!FFSTRCMP(FFL("ogg"),sfx))
       return sfx;
+#if 0 // [
+    else if (!FFSTRCMP(FFL("m4a"),sfx))
+      return sfx;
+    else if (!FFSTRCMP(FFL("aac"),sfx))
+      return sfx;
+#endif // ]
     else
       return default_sfx;
   }
@@ -215,7 +228,7 @@ static const ffchar_t *bg_track_out_sfx(const bg_track_t *track,
   else if (track->input.audio.ctx) {
     // we always transcode to FLAC.
     return FFL("flac");
-	}
+  }
   else {
 #if defined (BG_TRACK_AUDIO_SFX) // [
     return bg_track_audio_sfx(track,tree,FFL("mka"));
@@ -247,6 +260,7 @@ static const ffchar_t *bg_track_out_sfx(const bg_track_t *track,
 }
 
 #define BG_TRACK_BASENAME_TRACK
+#if 0 // [
 static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
     ffchar_t **opath, size_t len)
 {
@@ -262,12 +276,24 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
     const AVDictionaryEntry *track;
 #endif // ]
   } tag={
+#if 0 // [
     .title=av_dict_get(metadata,"TITLE",NULL,AV_DICT_IGNORE_SUFFIX),
 #if defined (BG_TRACK_BASENAME_TRACK) // [
     .track=av_dict_get(metadata,"TRACK",NULL,AV_DICT_IGNORE_SUFFIX),
+#endif // ]
+#else // ] [
+    .title=av_dict_get(metadata,"TITLE",NULL,0),
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+    .track=av_dict_get(metadata,"TRACK",NULL,0),
+#endif // ]
 #endif // ]
   };
 
+#if defined (BG_TRACK_ID) // [
+  int id=tag.track?atoi(tag.track->value):track->album.id;
+#else // ] [
+  int id=tag.track?atoi(tag.track->value):-1;
+#endif // ]
 #if defined (_WIN32) && ! defined (BG_WIN32_CREATE_LOCALE) // [
   const char *locale=NULL;
 #endif // ]
@@ -281,10 +307,8 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
     if (opath) {
       // swprintf() in any case writes a trailing '\0', hence (1+len).
 #if defined (BG_TRACK_BASENAME_TRACK) // [
-      if (tag.track) {
-        len=swprintf(*opath,1+track->target.pfx.len,L"%02d_",
-            atoi(tag.track->value));
-      }
+      if (0<id)
+        len=swprintf(*opath,1+track->target.pfx.len,L"%02d_",id);
       else
         len=0;
 #else // ] [
@@ -292,7 +316,7 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
           track->album.id<100?L"%02lu_":L"%lu_",track->album.id);
 #endif // ]
       wcscpy(*opath+track->target.pfx.len,track->target.title);
-      free(track->target.title);
+      PBU_FREE_GUARDED(track->target.title);
       track->target.title=NULL;
     }
     else {
@@ -312,7 +336,7 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
       track->target.title=malloc((len+1)*sizeof *track->target.title);
 
       if (!track->target.title) {
-        DMESSAGE("allocating intermediate path");
+        _DMESSAGE("allocating intermediate path");
         // set error.
         len=-1;
         goto e_path;
@@ -347,13 +371,17 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
       while (*op) {
         switch (*op) {
         case L'.':
+        case L',':
         case L'/':
         case L'\\':
         case L'(':
         case L')':
+        case L'[':
+        case L']':
         case L'&':
         case L':':
         case L' ':
+        case L'?':
         case L'\'':
           if (track->target.title<op&&'_'==op[-1])
             memmove(op,op+1,(wcslen(op+1)+1)*sizeof *op);
@@ -385,9 +413,8 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
 #endif // ]
 
 #if defined (BG_TRACK_BASENAME_TRACK) // [
-      if (tag.track) {
-        len=swprintf(NULL,0,L"%02d_",atoi(tag.track->value));
-      }
+      if (0<id)
+        len=swprintf(NULL,0,L"%02d_",id);
       else
         len=0;
 #else // ] [
@@ -403,10 +430,8 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
       // I.e. we assume enough memory is allocated for holding a
       // trailing '\0'.
 #if defined (BG_TRACK_BASENAME_TRACK) // [
-      if (tag.track) {
-        snprintf(*opath,1+track->target.pfx.len,"%02d_",
-            atoi(tag.track->value));
-      }
+      if (0<id)
+        snprintf(*opath,1+track->target.pfx.len,"%02d_",id);
 #else // ] [
       snprintf(*opath,1+track->target.pfx.len,
           track->album.id<100?"%02lu_":"%lu_",track->album.id);
@@ -414,7 +439,7 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
       // we convert the intermediate representation back into utf-8.
       wcstombs(*opath+track->target.pfx.len,track->target.title,
           len-track->target.pfx.len);
-      free(track->target.title);
+      PBU_FREE_GUARDED(track->target.title);
       track->target.title=NULL;
     }
     else {
@@ -425,7 +450,7 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
       track->target.title=malloc((len+1)*sizeof track->target.title[0]);
 
       if (!track->target.title) {
-        DMESSAGE("allocating intermediate path");
+        _DMESSAGE("allocating intermediate path");
         // set error.
         len=-1;
         goto e_path;
@@ -441,13 +466,17 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
       while (*op) {
         switch (*op) {
         case L'.':
+        case L',':
         case L'/':
         case L'\\':
         case L'(':
         case L')':
+        case L'[':
+        case L']':
         case L'&':
         case L':':
         case L' ':
+        case L'?':
         case L'\'':
           if (track->target.title<op&&'_'==op[-1])
             memmove(op,op+1,(wcslen(op+1)+1)*sizeof *op);
@@ -468,8 +497,8 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
         op[-1]=L'\0';
 
 #if defined (BG_TRACK_BASENAME_TRACK) // [
-      if (tag.track)
-        len=snprintf(NULL,0,"%02d_",atoi(tag.track->value));
+      if (0<id)
+        len=snprintf(NULL,0,"%02d_",id);
       else
         len=0;
 #else // ] [
@@ -505,6 +534,380 @@ static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
 e_path:
   return len;
 }
+#else // ] [
+#if defined (BG_UTF8_ITER) // [
+#if ! defined (_WIN32) // [
+#define BG_TITLE_NEXT_UTF8_ITER
+#endif // ]
+// TODO: consider
+//   https://www.gnu.org/software/libunistring/manual/libunistring.html
+#endif // ]
+#if 0 // [
+static int bg_title_next(const char *title, int special)
+{
+  int len=0;
+#if defined (BG_TITLE_NEXT_UTF8_ITER) // [
+  const char *next;
+  size_t size;
+  bg_utf8_iter_t i;
+#elif defined (_WIN32) // ] [
+  const char *next;
+#endif // ]
+
+#if ! defined (BG_TITLE_NEXT_UTF8_ITER) // ] [
+  while (*title) {
+#else // ] [
+  size=0;
+  bg_utf8_iter_first(&i,title,NULL,&size);
+
+  while (i.vmt->valid(&i)) {
+    i.vmt->apply(&i);
+
+    if (1==size) {
+#endif // ]
+      switch (*title) {
+      case '.':
+      case ',':
+      case '/':
+      case '\\':
+      case '(':
+      case ')':
+      case '[':
+      case ']':
+      case '&':
+      case ':':
+      case ' ':
+      case '?':
+      case '\'':
+        if (special)
+          return len;
+
+        break;
+      default:
+        if (!special)
+          return len;
+
+        break;
+      }
+#if defined (BG_TITLE_NEXT_UTF8_ITER) // [
+    }
+    else if (!special)
+      return len;
+
+    size=0;
+    next=(const char *)i.vmt->next(&i);
+    len+=next-title;
+    title=next;
+  }
+#else // ] [
+#if defined (_WIN32) // [
+    next=CharNextA(title);
+    len+=next-title;
+    title=next;
+#else // ] [
+    ++len;
+    ++title;
+#endif // ]
+  }
+#endif // ]
+
+  return len;
+}
+#else // ] [
+static int bg_title_next(const char *title, int special)
+{
+  int len=0;
+  const char *next;
+
+  while (*title) {
+    next=bg_char_nexta(title);
+
+    if (1==next-title) {
+      switch (*title) {
+      case '.':
+      case ',':
+      case '/':
+      case '\\':
+      case '(':
+      case ')':
+      case '[':
+      case ']':
+      case '&':
+      case ':':
+      case ' ':
+      case '?':
+      case '\'':
+        if (special)
+          return len;
+
+        break;
+      default:
+        if (!special)
+          return len;
+
+        break;
+      }
+    }
+    else if (!special)
+      return len;
+
+    len+=next-title;
+    title=next;
+  }
+
+  return len;
+}
+#endif // ]
+
+static int bg_title_length(char *title, wchar_t *wtitle, size_t leni)
+{
+  wchar_t *op=wtitle;
+  int leno=0;
+  int next,ch,len;
+
+  for (;;) {
+    if (!*title)
+      break;
+
+    ///////////////////////////////////////////////////////////////////////////
+    next=bg_title_next(title,0);
+    title+=next;
+
+    if (*title) {
+      if (next) {
+        if (wtitle) {
+          *wtitle++=L'_';
+          *wtitle=L'\0';
+          --leni;
+        }
+
+        ++leno;
+      }
+    }
+    else
+      return leno;
+
+    ///////////////////////////////////////////////////////////////////////////
+    next=bg_title_next(title,1);
+    ch=title[next];
+    title[next]=0;
+
+#if defined (_WIN32) // [
+    len=MultiByteToWideChar(
+      CP_UTF8,        // UINT                              CodePage,
+      0ul,            // DWORD                             dwFlags,
+      title,          // _In_NLS_string_(cbMultiByte)LPCCH lpMultiByteStr,
+      next,           // int                               cbMultiByte,
+      wtitle,         // LPWSTR                            lpWideCharStr,
+      leni            // int                               cchWideChar
+    );
+#else // ] [
+    len=mbstowcs(wtitle,title,leni);
+#endif // ]
+
+    if (wtitle) {
+      wtitle+=len;
+      *wtitle=L'\0';
+      leni-=len;
+    }
+
+    leno+=len;
+    title+=next;
+    *title=ch;
+  }
+
+  if (op) {
+#if defined (_WIN32) // [
+    _wcslwr(op);
+#else // ] [
+    while (*op) {
+#if 0 && defined (_WIN32) // [
+#if defined (BG_WIN32_CREATE_LOCALE) // [
+      *op=_towlower_l(*op,tree->param->locale);
+#else // ] [
+          // our system _create_locale() isn't availabe.
+      *op=towlower(*op);
+#endif // ]
+      op=CharNextW(op);
+#else // ] [
+      *op=towlower(*op);
+      ++op;
+#endif // ]
+    }
+#endif // ]
+  }
+
+  return leno;
+}
+
+static int bg_track_basename(bg_track_t *track, bg_tree_t *tree,
+    ffchar_t **opath, size_t len)
+{
+#if defined (_WIN32) && defined (BG_TRACK_BASENAME_TRACK) // [
+  enum { TRACK_SIZE=64 };
+#endif // ]
+  bg_param_t *param=tree->param;
+  AVDictionary *metadata=track->input.fmt.ctx->metadata;
+
+  struct {
+    const AVDictionaryEntry *title;
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+    const AVDictionaryEntry *track;
+#endif // ]
+  } tag={
+#if 0 // [
+    .title=av_dict_get(metadata,"TITLE",NULL,AV_DICT_IGNORE_SUFFIX),
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+    .track=av_dict_get(metadata,"TRACK",NULL,AV_DICT_IGNORE_SUFFIX),
+#endif // ]
+#else // ] [
+    .title=av_dict_get(metadata,"TITLE",NULL,0),
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+    .track=av_dict_get(metadata,"TRACK",NULL,0),
+#endif // ]
+#endif // ]
+  };
+
+#if defined (BG_TRACK_ID) // [
+  int id=tag.track?atoi(tag.track->value):track->album.id;
+#else // ] [
+  int id=tag.track?atoi(tag.track->value):-1;
+#endif // ]
+#if defined (_WIN32) && ! defined (BG_WIN32_CREATE_LOCALE) // [
+  const char *locale=NULL;
+#endif // ]
+  // NOTE: under Linux sizeof(wchar_t) is 4 bytes and under Windows
+  //   it is 2 bytes.
+#if 0 // [
+  wchar_t *op;
+#endif // ]
+  const ffchar_t *ipath,*sfx,*sp;
+
+  if ((BG_FLAGS_EXT_RENAME&param->flags.extension)&&tag.title) {
+#if defined (_WIN32) // [
+    if (opath) {
+      // swprintf() in any case writes a trailing '\0', hence (1+len).
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+      if (0<id)
+        len=swprintf(*opath,1+track->target.pfx.len,L"%02d_",id);
+      else
+        len=0;
+#else // ] [
+      swprintf(*opath,1+track->target.pfx.len,
+          track->album.id<100?L"%02lu_":L"%lu_",track->album.id);
+#endif // ]
+      wcscpy(*opath+track->target.pfx.len,track->target.title);
+      PBU_FREE_GUARDED(track->target.title);
+      track->target.title=NULL;
+    }
+    else {
+      // claculate the length of the intermediate representation.
+      len=bg_title_length(tag.title->value,NULL,0);
+
+      // allocate sufficient memory to hold the intermediate representatiion.
+      track->target.title=malloc((len+1)*sizeof *track->target.title);
+
+      if (!track->target.title) {
+        _DMESSAGE("allocating intermediate path");
+        // set error.
+        len=-1;
+        goto e_path;
+      }
+
+      // transform into the intermediate representation.
+      bg_title_length(tag.title->value,track->target.title,len);
+
+#if ! defined (BG_WIN32_CREATE_LOCALE) // [
+      // we need to switch back locale.
+      setlocale(LC_ALL,locale);
+#endif // ]
+
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+      if (0<id)
+        len=swprintf(NULL,0,L"%02d_",id);
+      else
+        len=0;
+#else // ] [
+      len=swprintf(NULL,0,
+          track->album.id<100?L"%02lu_":L"%lu_",track->album.id);
+#endif // ]
+      track->target.pfx.len=len;
+      len+=wcslen(track->target.title);
+    }
+#else // ] [
+    if (opath) {
+      // snprintf() in any case writes a trailing '\0', hence (1+len).
+      // I.e. we assume enough memory is allocated for holding a
+      // trailing '\0'.
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+      if (0<id)
+        snprintf(*opath,1+track->target.pfx.len,"%02d_",id);
+#else // ] [
+      snprintf(*opath,1+track->target.pfx.len,
+          track->album.id<100?"%02lu_":"%lu_",track->album.id);
+#endif // ]
+      // we convert the intermediate representation back into utf-8.
+      wcstombs(*opath+track->target.pfx.len,track->target.title,
+          len-track->target.pfx.len);
+      PBU_FREE_GUARDED(track->target.title);
+      track->target.title=NULL;
+    }
+    else {
+      // calculate the length for holding the intermediate representation.
+      len=bg_title_length(tag.title->value,NULL,0);
+
+      // allocate sufficient memory  in order to hold the intermediate
+      // representation.
+      track->target.title=malloc((len+1)*sizeof track->target.title[0]);
+
+      if (!track->target.title) {
+        _DMESSAGE("allocating intermediate path");
+        // set error.
+        len=-1;
+        goto e_path;
+      }
+
+      bg_title_length(tag.title->value,track->target.title,len);
+
+#if defined (BG_TRACK_BASENAME_TRACK) // [
+      if (0<id)
+        len=snprintf(NULL,0,"%02d_",id);
+      else
+        len=0;
+#else // ] [
+      len=snprintf(NULL,0,track->album.id<100?"%02lu_":"%lu_",track->album.id);
+#endif // ]
+      track->target.pfx.len=len;
+      // add the length for holding the title (transformed to lower case.)
+      len+=wcstombs(NULL,track->target.title,0);
+    }
+#endif // ]
+  }
+  else {
+    if (tree->source.basename) {
+      ipath=tree->source.basename;
+
+      if (opath)
+        memcpy(*opath,ipath,len*sizeof **opath);
+      else {
+        sfx=NULL;
+        sp=FFSTRSTR(ipath,FFL("."));
+
+        while (sp) {
+          sfx=sp;
+          sp=FFSTRSTR(++sp,FFL("."));
+        }
+
+        len=sfx?(size_t)(sfx-ipath):FFSTRLEN(ipath);
+      }
+    }
+    else
+      len=0;
+  }
+e_path:
+  return len;
+}
+#endif // ]
 
 ///////////////////////////////////////////////////////////////////////////////
 int bg_track_annotation_create(bg_tree_t *tree)
@@ -536,7 +939,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
     tree->target.path=tp=malloc(size*sizeof tp[0]);
 
     if (!tree->target.path) {
-      DMESSAGE("allocating output path");
+      _DMESSAGE("allocating output path");
       goto e_path;
     }
 
@@ -567,7 +970,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
     tree->target.path=tp=malloc(size*sizeof tp[0]);
 
     if (!tree->target.path) {
-      DMESSAGE("allocating output path");
+      _DMESSAGE("allocating output path");
       goto e_path;
     }
 
@@ -592,7 +995,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
   }
 
   if (!param->overwrite&&!ff_fcmp(tree->source.path,tree->target.path)) {
-    DMESSAGE("overwriting not permitted. Use option --overwrite");
+    _DMESSAGE("overwriting not permitted. Use option --overwrite");
     goto e_overwrite_target;
   }
 
@@ -601,14 +1004,15 @@ int bg_track_annotation_create(bg_tree_t *tree)
   tree->temp.path=tp=malloc((size+len2)*sizeof tp[0]);
 
   if (!tree->temp.path) {
-    DMESSAGE("allocating temporary path");
+    _DMESSAGE("allocating temporary path");
     goto e_temp;
   }
 
   memcpy(tp,tree->target.path,len1*sizeof tp[0]);
   tp+=len1;
 
-  if (FFPATHSEP!=tp[-1])
+  // Guarding by "len1" proposed by Hadrien Lacour <hadrien.lacour@posteo.net>.
+  if (len1&&FFPATHSEP!=tp[-1])
     *tp++=FFPATHSEP;
 
   tree->temp.basename=tp;
@@ -618,15 +1022,22 @@ int bg_track_annotation_create(bg_tree_t *tree)
 
   /////////////////////////////////////////////////////////////////////////////
   if (!ff_fcmp(tree->source.path,tree->temp.path)) {
-    DMESSAGE("attempt to overwrite source file");
+    _DMESSAGE("attempt to overwrite source file");
     goto e_overwrite_temp;
   }
 
   if (ff_fexists(tree->temp.path)) {
-    FFVMESSAGE("file \"%s\" exists; either remove it or by means of"
+#if defined (_WIN32) // [
+    _DMESSAGEV("file \"%S\" exists; either remove it or by means of"
+        " option --temp-prefix define a prefix for temporary files different"
+        " from \"%S\"",
+        tree->temp.path,param->temp_prefix);
+#else // ] [
+    _DMESSAGEV("file \"%s\" exists; either remove it or by means of"
         " option --temp-prefix define a prefix for temporary files different"
         " from \"%s\"",
         tree->temp.path,param->temp_prefix);
+#endif // ]
     goto e_temp_exists;
   }
 
@@ -640,7 +1051,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
     track->target.oem.basename=bg_wcs2str(tree->target.basename,CP_OEMCP);
 
     if (!track->target.oem.basename) {
-      DMESSAGE("creating oem basename");
+      _DMESSAGE("creating oem basename");
       goto e_basename;
     }
   }
@@ -652,7 +1063,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
   track->target.utf8.path=bg_wcs2str(tree->target.path,CP_UTF8);
   
   if (!track->target.utf8.path) {
-    DMESSAGE("creating utf-8 representation of path");
+    _DMESSAGE("creating utf-8 representation of path");
     goto e_patha;
   }
 
@@ -661,7 +1072,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
   track->target.path=bg_wcs2str(tree->target.path,CP_UTF8);
   
   if (!track->target.path) {
-    DMESSAGE("creating utf-8 representation of path");
+    _DMESSAGE("creating utf-8 representation of path");
     goto e_patha;
   }
 
@@ -672,7 +1083,7 @@ int bg_track_annotation_create(bg_tree_t *tree)
   track->temp.path=bg_wcs2str(tree->temp.path,CP_UTF8);
   
   if (!track->temp.path) {
-    DMESSAGE("creating utf-8 representation of temporary path");
+    _DMESSAGE("creating utf-8 representation of temporary path");
     goto e_tempa;
   }
 #endif // ]
@@ -680,44 +1091,52 @@ int bg_track_annotation_create(bg_tree_t *tree)
   return 0;
 //cleanup:
 #if defined (_WIN32) // [
-  free(track->temp.path);
+  PBU_FREE_GUARDED(track->temp.path);
 e_tempa:
 #if defined (BG_WIN32_TARGET_UTF8) // [
-  free(track->target.utf8.path);
+  PBU_FREE_GUARDED(track->target.utf8.path);
 #else // ] [
-  free(track->target.path);
+  PBU_FREE_GUARDED(track->target.path);
 #endif // ]
 e_patha:
   if (track->target.oem.basename)
-    free(track->target.oem.basename);
+    PBU_FREE_GUARDED(track->target.oem.basename);
 e_basename:
 #endif // ]
 e_temp_exists:
 e_overwrite_temp:
-  free(tree->temp.path);
+  PBU_FREE_GUARDED(tree->temp.path);
 e_temp:
 e_overwrite_target:
-  free(tree->target.path);
+  PBU_FREE_GUARDED(tree->target.path);
 e_path:
   return -1;
 }
 
+FFUNUSED static void bg_PBU_FREE_GUARDED_guarded(void **p)
+{
+  if (*p) {
+    PBU_FREE_GUARDED(*p);
+    *p=NULL;
+  }
+}
+
 void bg_track_annotation_destroy(bg_tree_t *tree FFUNUSED)
 {
 #if defined (_WIN32) // [
   bg_track_t *track=&tree->track;
 
-  free(track->temp.path);
+  PBU_FREE_GUARDED(track->temp.path);
 #if defined (BG_WIN32_TARGET_UTF8) // [
-  free(track->target.utf8.path);
+  PBU_FREE_GUARDED(track->target.utf8.path);
 #else // ] [
-  free(track->target.path);
+  PBU_FREE_GUARDED(track->target.path);
 #endif // ]
   if (track->target.oem.basename)
-    free(track->target.oem.basename);
+    PBU_FREE_GUARDED(track->target.oem.basename);
 #endif // ]
-  free(tree->temp.path);
-  free(tree->target.path);
+  PBU_FREE_GUARDED(tree->temp.path);
+  PBU_FREE_GUARDED(tree->target.path);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -744,7 +1163,7 @@ int bg_album_annotation_create(bg_tree_t *tree)
       tree->target.path=tp=malloc(size*sizeof tp[0]);
 
       if (!tree->target.path) {
-        DMESSAGE("allocating output path");
+        _DMESSAGE("allocating output path");
         goto e_path;
       }
 
@@ -768,8 +1187,8 @@ int bg_album_annotation_create(bg_tree_t *tree)
 
       /////////////////////////////////////////////////////////////////////////
       if (!tree->param->overwrite&&ff_mkdir(tree->target.path)<0) {
-  	    DMESSAGE("creating directory");
-  	    goto e_mkdir;
+        _DMESSAGE("creating directory");
+        goto e_mkdir;
       }
     }
     else {
@@ -790,7 +1209,7 @@ success:
 e_mkdir:
 #endif // ]
   if (tree->target.path)
-    free(tree->target.path);
+    PBU_FREE_GUARDED(tree->target.path);
 e_path:
   return -1;
 }
@@ -798,7 +1217,7 @@ e_path:
 void bg_album_annotation_destroy(bg_tree_t *tree FFUNUSED)
 {
   if (tree->target.path)
-    free(tree->target.path);
+    PBU_FREE_GUARDED(tree->target.path);
 }
 
 
@@ -811,7 +1230,7 @@ int bg_root_annotation_create(bg_tree_t *tree)
 
     if (!tree->param->overwrite&&ff_mkdir(tree->target.path)<0) {
       /////////////////////////////////////////////////////////////////////////
-  	  DMESSAGE("creating directory");
+      _DMESSAGE("creating directory");
       goto e_mkdir;
     }
 
