@@ -1,7 +1,7 @@
 /*
  * bg_muxer.c
  *
- * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
+ * Copyright (C) 2019 Peter Belkner <info@pbelkner.de>
  * Nanos gigantum humeris insidentes #TeamWhite
  *
  * This program is free software; you can redistribute it and/or
@@ -30,8 +30,8 @@
 #include <bg.h>
 
 ///////////////////////////////////////////////////////////////////////////////
-static bg_visitor_vmt_t bg_muxer_vmt;
 static ff_output_callback_t bg_output_callback;
+static bg_visitor_vmt_t bg_muxer_vmt;
 
 ///////////////////////////////////////////////////////////////////////////////
 int bg_muxer_create(bg_visitor_t *vis)
@@ -63,6 +63,7 @@ static int bg_muxer_copy_file(ffchar_t *source, ffchar_t *target)
   );
 
   err=0;
+
 #else // ] [
   // https://stackoverflow.com/questions/2180079/how-can-i-copy-a-file-on-unix-using-c
   struct {
@@ -75,7 +76,15 @@ static int bg_muxer_copy_file(ffchar_t *source, ffchar_t *target)
   fd.source=open(source,O_RDONLY);
 
   if (fd.source<0) {
-    DVMESSAGE("opening source \"%s\"",source);
+#if 0 // [
+#if defined (_WIN32) // [
+    _DMESSAGEV("opening source \"%S\"",source);
+#else // ] [
+    _DMESSAGEV("opening source \"%s\"",source);
+#endif // ]
+#else // ] [
+    _DMESSAGEV("opening source \"%" PBU_PRIs "\"",source);
+#endif // ]
     goto e_source;
   }
 
@@ -83,7 +92,15 @@ static int bg_muxer_copy_file(ffchar_t *source, ffchar_t *target)
   fd.target=open(target,O_WRONLY|O_CREAT|O_EXCL,0666);
 
   if (fd.target<0) {
-    DVMESSAGE("opening target \"%s\"",target);
+#if 0 // [
+#if defined (_WIN32) // [
+    _DMESSAGEV("opening target \"%S\"",target);
+#else // ] [
+    _DMESSAGEV("opening target \"%s\"",target);
+#endif // ]
+#else // ] [
+    _DMESSAGEV("opening target \"%" PBU_PRIs "\"",target);
+#endif // ]
     goto e_target;
   }
 
@@ -99,7 +116,7 @@ static int bg_muxer_copy_file(ffchar_t *source, ffchar_t *target)
       written=write(fd.target,bp,size);
 
       if (written<=0) {
-        DMESSAGE("copying");
+        _DMESSAGE("copying");
         goto e_copy;
       }
 
@@ -121,7 +138,8 @@ e_source:
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-static int bg_muxer_dispatch_file(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
+#if defined (BG_PARAM_THREADS) // [
+static int bg_muxer_file(bg_tree_t *tree, FFUNUSED bg_visitor_t *vis)
 {
   int err=-1;
   bg_param_t *param=tree->param;
@@ -134,13 +152,13 @@ static int bg_muxer_dispatch_file(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
   // file annotation is created in each case.
   if (param->output.dirname||param->overwrite) {
     if (tree->vmt->annotation.create(tree)<0) {
-      DMESSAGE("annotating");
+      _DMESSAGE("annotating");
       goto e_annotate;
     }
   }
 
   if (bg_muxer_copy_file(tree->source.path,tree->temp.path)<0) {
-    DMESSAGE("copying");
+    _DMESSAGE("copying");
     goto e_copy;
   }
 
@@ -156,10 +174,264 @@ e_copy:
 e_annotate:
   return err;
 }
+#endif // ]
+
+static int bg_muxer_dispatch_file(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
+{
+#if defined (BG_PARAM_THREADS) // [
+  int err=-1;
+
+  if (tree->param->nthreads) {
+#if defined (BG_PARAM_SCRIPT) // [
+    bg_param_threads_visitor_run(&tree->param->threads,tree->param->script,
+        vis,tree,bg_muxer_file);
+#else // ] [
+    bg_param_threads_visitor_run(&tree->param->threads,vis,tree,
+        bg_muxer_file);
+#endif // ]
+    err=0;
+  }
+  else
+    err=bg_muxer_file(tree,vis);
+
+  return err;
+#else // ] [
+  int err=-1;
+  bg_param_t *param=tree->param;
+
+  if (!(BG_FLAGS_EXT_COPY&tree->param->flags.extension))
+    goto success;
+  else if (tree->param->overwrite)
+    goto success;
+
+  // file annotation is created in each case.
+  if (param->output.dirname||param->overwrite) {
+    if (tree->vmt->annotation.create(tree)<0) {
+      _DMESSAGE("annotating");
+      goto e_annotate;
+    }
+  }
+
+  if (bg_muxer_copy_file(tree->source.path,tree->temp.path)<0) {
+    _DMESSAGE("copying");
+    goto e_copy;
+  }
+
+  ff_mv(tree->temp.path,tree->target.path);
+
+  /////////////////////////////////////////////////////////////////////////////
+success:
+  err=0;
+//cleanup:
+e_copy:
+  if (param->output.dirname||param->overwrite)
+    tree->vmt->annotation.destroy(tree);
+e_annotate:
+  return err;
+#endif // ]
+}
+
+#if defined (BG_PARAM_THREADS) // [
+static int bg_muxer_track(bg_tree_t *tree, bg_visitor_t *vis)
+{
+  int err=-1;
+  bg_track_t *track=&tree->track;
+  bg_tree_t *parent=tree->parent;
+  bg_param_t *param=tree->param;
+  int apply=BG_FLAGS_MODE_APPLY&param->flags.mode;
+#if defined (FF_FLAC_HACK) // [
+  int hack;
+  AVCodecParameters *codecpar;
+  enum AVSampleFormat sample_fmt;
+#endif // ]
+  ff_inout_t output;
+  double gain_track,gain_album,gain,volume;
+  char filter[128];
+  ff_muxer_t muxer;
+
+  /////////////////////////////////////////////////////////////////////////////
+  ++vis->depth; // [
+
+  /////////////////////////////////////////////////////////////////////////////
+//_DWRITELNV("\"%s\"",tree->source.path);
+#if defined (FF_FLAC_HACK) // [
+  if (ff_input_open_muxer(&track->input,&hack)<0) {
+    _DMESSAGE("re-opening input");
+    goto e_input;
+  }
+#else // ] [
+  if (ff_input_open_muxer(&track->input)<0) {
+    _DMESSAGE("re-opening input");
+    goto e_input;
+  }
+#endif // ]
+
+//_DWRITELNV("\"%s\"",tree->source.path);
+  /////////////////////////////////////////////////////////////////////////////
+  // track annotation is created in each case.
+  if (param->output.dirname||param->overwrite) {
+    if (tree->vmt->annotation.create(tree)<0) {
+      _DMESSAGE("annotating");
+      goto e_annotate;
+    }
+  }
+
+//_DWRITELNV("\"%s\"",tree->source.path);
+  /////////////////////////////////////////////////////////////////////////////
+#if defined (BG_PARAM_QUIET) // [
+  if (!param->quiet) {
+#endif // ]
+#if defined (BG_PARAM_SKIP_SCAN) // [
+    if (param->skip_scan)
+      _FPRINTFV(stdout,"%s\n",bg_tree_out_basename(tree));
+    else {
+#endif // ]
+      _FPRINTFV(stdout,"[%lu/%lu] %s\n",track->root.id,param->count.max,
+          bg_tree_out_basename(tree));
+#if defined (BG_PARAM_SKIP_SCAN) // [
+    }
+#endif // ]
+
+    fflush(stdout);
+#if defined (BG_PARAM_QUIET) // [
+  }
+#endif // ]
+
+//_DWRITELNV("\"%s\"",tree->source.path);
+  /////////////////////////////////////////////////////////////////////////////
+#if defined (FF_FLAC_HACK) // [
+  // as it seems it is impossible to pass through a partial flac stream
+  // (cf. "https://trac.ffmpeg.org/ticket/7864".) our hack consists in
+  // reading/decoding and re-encoding/writing./ hence we've need to tell
+  // the output the format from decoding.
+  codecpar=track->input.fmt.ctx->streams[track->input.ai]->codecpar;
+  sample_fmt=hack?codecpar->format:AV_SAMPLE_FMT_NONE;
+
+  if (ff_output_create(&output,&bg_output_callback,tree,sample_fmt)<0) {
+    _DMESSAGE("creating output");
+    goto e_output;
+  }
+#else // ] [
+  if (ff_output_streams_create(&output)<0) {
+    _DMESSAGE("creating output streams");
+    goto e_output_streams;
+  }
+#endif // ]
+
+//_DWRITELNV("\"%s\"",tree->source.path);
+  /////////////////////////////////////////////////////////////////////////////
+#if defined (FF_FLAC_HACK) // [
+  apply=apply&&!hack;
+#endif // ]
+
+  if (apply) {
+    if (param->weight.enabled) {
+      gain_album=param->norm-lib1770_stats_get_mean(parent->stats.momentary,
+          param->momentary.mean.gate);
+      gain_track=param->norm-lib1770_stats_get_mean(tree->stats.momentary,
+          param->momentary.mean.gate);
+      gain=gain_album+param->weight.value*(gain_track-gain_album);
+    }
+    else {
+      gain=param->norm-lib1770_stats_get_mean(parent->stats.momentary,
+          param->momentary.mean.gate);
+    }
+
+    volume=LIB1770_DB2Q(gain);
+    snprintf(filter,sizeof filter,"volume=%1.2f",volume);
+  }
+
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+//_DWRITELNV("\"%s\"",filter);
+  ///////////////////////////////////////////////////////////////////////////
+  if (ff_muxer_create(&muxer,&track->input,&output,apply?filter:NULL)<0) {
+    _DMESSAGE("creating muxer");
+    goto e_muxer;
+  }
+
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+  /////////////////////////////////////////////////////////////////////////////
+  if (ff_muxer_loop(&muxer)<0) {
+    _DMESSAGE("re-encoder looping");
+    goto e_loop;
+  }
+
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+  /////////////////////////////////////////////////////////////////////////////
+  err=0;
+//cleanup:
+e_loop:
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+  ff_muxer_destroy(&muxer);
+e_muxer:
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+  ff_output_destroy(&output);
+e_output:
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+  if (err)
+    ff_rm(tree->temp.path);
+  else {
+    // before moving/renaming the newley created file we first must have
+    // closed the muxer/output/input!!!
+//_DWRITELNV("\"%" PBU_PRIs "\"",tree->source.path);
+    ff_input_close(&track->input);
+
+#if defined (BG_PARAM_SLEEP) // [
+    if (0<param->sleep) {
+#if defined (_WIN32) // [
+      Sleep(param->sleep);
+#else // ] [
+      sleep(param->sleep);
+#endif // ]
+    }
+#endif // ]
+
+//_DWRITELNV("\"%s\"",tree->source.path);
+    err=ff_mv(tree->temp.path,tree->target.path);
+//_DWRITELNV("\"%s\"",tree->source.path);
+
+    if (err<0)
+      _DMESSAGE("moving");
+  }
+
+//_DWRITELNV("\"%s\"",tree->source.path);
+  if (param->output.dirname||param->overwrite)
+    tree->vmt->annotation.destroy(tree);
+e_annotate:
+//_DWRITELNV("\"%s\"",tree->source.path);
+  ff_input_close(&track->input);
+e_input:
+  --vis->depth; // ]
+//_DWRITELNV("\"%s\" err:%d",tree->source.path,err);
+  return err;
+}
+#endif // ]
 
 static int bg_muxer_dispatch_track(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
 {
   int err=-1;
+#if defined (BG_PARAM_THREADS) // [
+  if (tree->param->nthreads) {
+#if defined (BG_PARAM_SCRIPT) // [
+    bg_param_threads_visitor_run(&tree->param->threads,tree->param->script,
+        vis,tree,bg_muxer_track);
+#else // ] [
+    bg_param_threads_visitor_run(&tree->param->threads,vis,tree,
+        bg_muxer_track);
+#endif // ]
+    err=0;
+  }
+  else {
+#if defined (BG_PARAM_SCRIPT) // [
+    if (tree->param->script)
+      err=bg_process_tree_run_script(tree);
+    else
+#endif // ]
+      err=bg_muxer_track(tree,vis);
+
+    return err;
+  }
+#else // ] [
   bg_track_t *track=&tree->track;
   bg_tree_t *parent=tree->parent;
   bg_param_t *param=tree->param;
@@ -176,14 +448,14 @@ static int bg_muxer_dispatch_track(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
   ff_muxer_t muxer;
 
   /////////////////////////////////////////////////////////////////////////////
-#if defined (FF_FLAC_HACK) // ] [
+#if defined (FF_FLAC_HACK) // [
   if (ff_input_open_muxer(&track->input,&hack)<0) {
-    DMESSAGE("re-opening input");
+    _DMESSAGE("re-opening input");
     goto e_input;
   }
 #else // ] [
   if (ff_input_open_muxer(&track->input)<0) {
-    DMESSAGE("re-opening input");
+    _DMESSAGE("re-opening input");
     goto e_input;
   }
 #endif // ]
@@ -192,15 +464,30 @@ static int bg_muxer_dispatch_track(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
   // track annotation is created in each case.
   if (param->output.dirname||param->overwrite) {
     if (tree->vmt->annotation.create(tree)<0) {
-      DMESSAGE("annotating");
+      _DMESSAGE("annotating");
       goto e_annotate;
     }
   }
 
   /////////////////////////////////////////////////////////////////////////////
-  fprintf(stdout,"[%lu/%lu] %s\n",track->root.id,param->count.max,
-      bg_tree_out_basanamen(tree));
-  fflush(stdout);
+#if defined (BG_PARAM_QUIET) // [
+  if (!param->quiet) {
+#endif // ]
+#if defined (BG_PARAM_SKIP_SCAN) // [
+    if (param->skip_scan)
+      fprintf(stdout,"%s\n",bg_tree_out_basename(tree));
+    else {
+#endif // ]
+      fprintf(stdout,"[%lu/%lu] %s\n",track->root.id,param->count.max,
+          bg_tree_out_basename(tree));
+#if defined (BG_PARAM_SKIP_SCAN) // [
+    }
+#endif // ]
+
+    fflush(stdout);
+#if defined (BG_PARAM_QUIET) // [
+  }
+#endif // ]
 
   /////////////////////////////////////////////////////////////////////////////
 #if defined (FF_FLAC_HACK) // [
@@ -212,12 +499,12 @@ static int bg_muxer_dispatch_track(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
   sample_fmt=hack?codecpar->format:AV_SAMPLE_FMT_NONE;
 
   if (ff_output_create(&output,&bg_output_callback,tree,sample_fmt)<0) {
-    DMESSAGE("creating output");
+    _DMESSAGE("creating output");
     goto e_output;
   }
 #else // ] [
   if (ff_output_streams_create(&output)<0) {
-    DMESSAGE("creating output streams");
+    _DMESSAGE("creating output streams");
     goto e_output_streams;
   }
 #endif // ]
@@ -246,13 +533,13 @@ static int bg_muxer_dispatch_track(bg_visitor_t *vis FFUNUSED, bg_tree_t *tree)
 
   ///////////////////////////////////////////////////////////////////////////
   if (ff_muxer_create(&muxer,&track->input,&output,apply?filter:NULL)<0) {
-    DMESSAGE("creating muxer");
+    _DMESSAGE("creating muxer");
     goto e_muxer;
   }
 
   /////////////////////////////////////////////////////////////////////////////
   if (ff_muxer_loop(&muxer)<0) {
-    DMESSAGE("re-encoder looping");
+    _DMESSAGE("re-encoder looping");
     goto e_loop;
   }
 
@@ -284,7 +571,7 @@ e_output:
     err=ff_mv(tree->temp.path,tree->target.path);
 
     if (err<0)
-      DMESSAGE("moving");
+      _DMESSAGE("moving");
   }
 
   if (param->output.dirname||param->overwrite)
@@ -292,6 +579,7 @@ e_output:
 e_annotate:
   ff_input_close(&track->input);
 e_input:
+#endif // ]
   return err;
 }
 
@@ -300,14 +588,49 @@ static int bg_muxer_dispatch_album(bg_visitor_t *vis, bg_tree_t *tree)
   int err=-1;
   bg_tree_t *cur;
 
+#if defined (BG_PARAM_THREADS) // [
+  /////////////////////////////////////////////////////////////////////////////
+  tree->helper.nchildren=tree->album.nchildren.cur;
+_DWRITELNV(">>> %u (%u %p)",tree->helper.nchildren,tree->album.nchildren.cur,tree->album.first);
+#endif // ]
+
   /////////////////////////////////////////////////////////////////////////////
   for (cur=tree->album.first;cur;cur=cur->next) {
+#if 0 // [
+#if defined (_WIN32) // [
+_DWRITELNV("    * \"%S\"",cur->source.path);
+#else // ] [
+_DWRITELNV("    * \"%s\"",cur->source.path);
+#endif // ]
+#else // ] [
+_DWRITELNV("    * \"%" PBU_PRIs "\"",cur->source.path);
+#endif // ]
     if (cur->vmt->accept(cur,vis)<0) {
-      DMESSAGE("child accepting muxer");
+      _DMESSAGE("child accepting muxer");
       goto e_child;
     }
   }
 
+#if defined (BG_PARAM_THREADS) // [
+  /////////////////////////////////////////////////////////////////////////////
+  if (tree->param->nthreads) {
+    bg_sync_lock(&tree->helper.sync); // {
+
+    while (tree->helper.nchildren)
+      bg_sync_wait(&tree->helper.sync);
+
+    bg_sync_unlock(&tree->helper.sync); // }
+
+#if defined (BG_VISITOR_NOTIFY_PARENT) // [
+    if (tree->parent) {
+      bg_sync_lock(&tree->helper.sync); // {
+      bg_sync_signal(&tree->helper.sync);
+      bg_sync_unlock(&tree->helper.sync); // }
+    }
+#endif // ]
+  }
+#endif // ]
+
   /////////////////////////////////////////////////////////////////////////////
   err=0;
 //cleanup:
@@ -323,7 +646,7 @@ static int bg_muxer_dispatch_root(bg_visitor_t *vis, bg_tree_t *tree)
   /////////////////////////////////////////////////////////////////////////////
   for (cur=tree->album.first;cur;cur=cur->next) {
     if (cur->vmt->accept(cur,vis)<0) {
-      DMESSAGE("accepting visitor");
+      _DMESSAGE("accepting visitor");
       goto e_child;
     }
   }
@@ -410,13 +733,13 @@ static void bg_tag(void *data, bg_bits_t *bits, const char *key,
   double gain;
 
   if (!key) {
-    DMESSAGE("missing key");
+    _DMESSAGE("missing key");
     return;
   }
 
   if (STRNCASECMP(param->tag.pfx,key,len)) {
     if (!ivalue) {
-      DMESSAGE("missing value");
+      _DMESSAGE("missing value");
       return;
     }
 
@@ -601,17 +924,17 @@ static enum AVCodecID bg_muxer_codec_id(const void *data,
     codec=avcodec_find_encoder_by_name(param->codec.name);
 
     if (!codec) {
-      DVWARNING("audio codec \"%s\" does not exist;"
+      _DWARNINGV("audio codec \"%s\" does not exist;"
           " falling back to \"FLAC\"",param->codec.name);
       return AV_CODEC_ID_FLAC;
     }
     else if (codec->id<AV_CODEC_ID_FIRST_AUDIO) {
-      DVWARNING("codec \"%s\" is not an audio codec;"
+      _DWARNINGV("codec \"%s\" is not an audio codec;"
           " falling back to \"FLAC\"",param->codec.name);
       return AV_CODEC_ID_FLAC;
     }
     else if (AV_CODEC_ID_FIRST_SUBTITLE<=codec->id) {
-      DVWARNING("codec \"%s\" is not an audio codec;"
+      _DWARNINGV("codec \"%s\" is not an audio codec;"
           " falling back to \"FLAC\"",param->codec.name);
       return AV_CODEC_ID_FLAC;
     }
