@@ -1,7 +1,7 @@
 /*
  * bg_track.c
  *
- * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
+ * Copyright (C) 2019 Peter Belkner <info@pbelkner.de>
  * Nanos gigantum humeris insidentes #TeamWhite
  *
  * This program is free software; you can redistribute it and/or
@@ -38,7 +38,7 @@ int bg_track_content_create(bg_tree_t *tree)
   // which in front already has called bg_tree_common_create() and just
   // leaves us with setting the vmt!
   if (bg_tree_common_create(tree,param,param,path,&bg_track_vmt)<0) {
-    DMESSAGE("creating tree");
+    _DMESSAGE("creating tree");
     goto etree;
   }
 #else // ] [
@@ -55,7 +55,11 @@ int bg_track_content_create(bg_tree_t *tree)
   err=ff_input_create(&track->input,&bg_input_callback,tree,1);
 
   if (err<0) {
-    FFVWARNING(FFL("opening track \"%s\""),pathw);
+#if defined (_WIN32) // [
+    _DWARNINGV("opening track \"%S\"",pathw);
+#else // ] [
+    _DWARNINGV("opening track \"%s\"",path);
+#endif // ]
     goto e_input;
   }
 #else // ] [
@@ -67,7 +71,7 @@ int bg_track_content_create(bg_tree_t *tree)
 
   /////////////////////////////////////////////////////////////////////////////
   if (!track->input.audio.ctx->channel_layout) {
-    DMESSAGE("missing input channel layout");
+    _DMESSAGE("missing input channel layout");
     goto e_channel_layout;
   }
 
@@ -75,16 +79,18 @@ int bg_track_content_create(bg_tree_t *tree)
   track->root.id=++param->count.cur;
 
   /////////////////////////////////////////////////////////////////////////////
+#if defined (BG_PARAM_QUIET) // [
+  if (!param->quiet&&!param->process&&!param->suppress.progress) {
+#else // ] [
   if (!param->process&&!param->suppress.progress) {
-    ff_printer_reset(p);
-
-#if defined (_WIN32) // [
-    if (stdout==p->f||stderr==p->f)
-      ff_printer_wprintf(p,L"%d",track->root.id);
-    else
 #endif // ]
-      ff_printer_printf(p,"%d",track->root.id);
+    ff_printer_reset(p);
+    _FF_PRINTER_PRINTF(p,"%d",track->root.id);
+#if ! defined (BG_PARAM_QUIET) // [
   }
+#else // ] [
+  }
+#endif // ]
 
   ++tree->parent->album.nleafs;
   ff_input_close(&track->input);
@@ -120,10 +126,16 @@ static int bg_track_accept(bg_tree_t *tree, bg_visitor_t *vis)
   return vis->vmt->dispatch_track(vis,tree);
 }
 
-static bg_tree_vmt_t bg_track_vmt={
-#if defined (PBU_DEBUG) // [
-  .id=FFL("track"),
+#if defined (BG_TRACK_ID) // [
+static void bg_track_track_id(bg_tree_t *tree, int *id)
+{
+  if (id)
+    tree->track.album.id=++*id;
+}
 #endif // ]
+
+static bg_tree_vmt_t bg_track_vmt={
+  .id=FFL("track"),
   .type=BG_TREE_TYPE_TRACK,
   .destroy=bg_track_destroy,
   .accept=bg_track_accept,
@@ -131,6 +143,9 @@ static bg_tree_vmt_t bg_track_vmt={
     .create=bg_track_annotation_create,
     .destroy=bg_track_annotation_destroy,
   },
+#if defined (BG_TRACK_ID) // [
+  .track_id=bg_track_track_id,
+#endif // ]
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -174,7 +189,13 @@ static int input_csv(const void *data)
 
 static int input_suppress_progress(const void *data)
 {
+#if defined (BG_PARAM_QUIET) // [
+  bg_param_t *param=((const bg_tree_t *)data)->param;
+
+  return param->quiet||param->suppress.progress;
+#else // ] [
   return ((const bg_tree_t *)data)->param->suppress.progress;
+#endif // ]
 }
 
 static int64_t input_interval_begin(const void *data)
@@ -195,19 +216,16 @@ static int input_stats_create(void *data, const AVCodecParameters *codecpar)
 {
   bg_tree_t *tree=data;
   bg_track_t *track=&tree->track;
+  int channels=codecpar->channels<FF_LFE_THRESHOLD
+      ?codecpar->channels
+      :codecpar->channels-1;
 
   /////////////////////////////////////////////////////////////////////////////
   if (tree->stats.momentary||tree->stats.shortterm) {
-#if defined (LIB1770_LFE) // [
-    track->filter.pre=lib1770_pre_new_lfe(codecpar->sample_rate,
-        codecpar->channels,tree->param->lfe);
-#else // ] [
-    track->filter.pre=lib1770_pre_new(codecpar->sample_rate,
-        codecpar->channels);
-#endif // ]
+    track->filter.pre=lib1770_pre_new(codecpar->sample_rate,channels);
 
     if (!track->filter.pre) {
-      DMESSAGE("creating pre-filter");
+      _DMESSAGE("creating pre-filter");
       goto epre;
     }
   }
@@ -218,7 +236,7 @@ static int input_stats_create(void *data, const AVCodecParameters *codecpar)
         tree->param->momentary.ms,tree->param->momentary.partition);
 
     if (!track->block.momentary) {
-      DMESSAGE("creating momentary block");
+      _DMESSAGE("creating momentary block");
       goto emomentary;
     }
 
@@ -234,7 +252,7 @@ static int input_stats_create(void *data, const AVCodecParameters *codecpar)
         tree->param->shortterm.ms,tree->param->shortterm.partition);
 
     if (!track->block.shortterm) {
-      DMESSAGE("creating shortterm block");
+      _DMESSAGE("creating shortterm block");
       goto eshortterm;
     }
 
@@ -282,6 +300,7 @@ static int input_stats_add(void *data, int upsampled, AVFrame *frame)
 
   if (frame) {
     for (ff_iter_first(&i,frame);i.vmt->valid(&i);i.vmt->next(&i)) {
+//DWRITELN("{");
       if (upsampled) {
 #if defined (FF_UPSAMPLE_MODIFYX) // [
         i.vmt->norm(&i,NULL,&tree->stats.truepeak,upsampled);
@@ -290,12 +309,15 @@ static int input_stats_add(void *data, int upsampled, AVFrame *frame)
 #endif // ]
       }
       else if (track->filter.pre) {
+//DWRITELN(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
 #if defined (FF_UPSAMPLE_MODIFYX) // [
         i.vmt->norm(&i,sample,&tree->stats.samplepeak,upsampled);
 #else // ] [
         i.vmt->norm(&i,sample,&tree->stats.samplepeak);
 #endif // ]
+//DWRITELN("================================================");
         lib1770_pre_add_sample(track->filter.pre,sample);
+//DWRITELN("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
       }
       else {
 #if defined (FF_UPSAMPLE_MODIFYX) // [
@@ -304,6 +326,7 @@ static int input_stats_add(void *data, int upsampled, AVFrame *frame)
         i.vmt->norm(&i,NULL,&tree->stats.samplepeak);
 #endif // ]
       }
+//DWRITELN("}");
     }
   }
 #if ! defined (FF_INPUT_CALLBACK_FLUSH) // [
