@@ -1,7 +1,7 @@
 /*
  * ff.h
  *
- * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
+ * Copyright (C) 2019 Peter Belkner <info@pbelkner.de>
  * Nanos gigantum humeris insidentes #TeamWhite
  *
  * This program is free software; you can redistribute it and/or
@@ -41,6 +41,10 @@ extern "C" {
 #endif // ]
 
 ///////////////////////////////////////////////////////////////////////////////
+#if defined (PBU_CONSOLE_UTF16) // [
+#define FF_CONSOLE_UTF16
+#endif // ]
+
 #if defined (_WIN32) // [
 #define FF_OPTION_UTF16
 #define FF_GETOPT_LONG(argc,argv,optstring,longopts,longindex) \
@@ -60,12 +64,14 @@ extern "C" {
 #define FFSTRCPY(s1,s2) wcscpy(s1,s2)
 #define FFSTRSTR(s1,s2) wcsstr(s1,s2)
 #define FFFOPEN(path,mode) _wfopen(path,mode)
+#if 0 // [
 #define _FFPUTC(c,f) fputwc(c,f)
 #define FFPUTC(c,f) _FFPUTC(L##c,f)
 #define _FFPUTS(s,f) fputws(s,f)
 #define FFPUTS(s,f) _FFPUTS(L##s,f)
 #define _FFPRINTF(f,format,...) fwprintf(f,format,__VA_ARGS__)
 #define FFPRINTF(f,format,...) _FFPRINTF(f,L##format,__VA_ARGS__)
+#endif // ]
 #define FFSPRINTF(s,size,format,...) swprintf(s,size,format,__VA_ARGS__)
 #define FFSNPRINTF(s,size,format,...) snwprintf(s,size,format,__VA_ARGS__)
 #define FFMESSAGE(m) PBU_DMESSAGEW(L##m)
@@ -100,12 +106,14 @@ typedef wchar_t ffchar_t;
 #define FFSTRCPY(s1,s2) strcpy(s1,s2)
 #define FFSTRSTR(s1,s2) strstr(s1,s2)
 #define FFFOPEN(path,mode) fopen(path,mode)
+#if 0 // [
 #define _FFPUTC(s,f) fputc(s,f)
 #define FFPUTC(s,f) _FFPUTC(s,f)
 #define _FFPUTS(s,f) fputs(s,f)
 #define FFPUTS(s,f) _FFPUTS(s,f)
 #define _FFPRINTF(f,format,...) fprintf(f,format,__VA_ARGS__)
 #define FFPRINTF(f,format,...) _FFPRINTF(f,format,__VA_ARGS__)
+#endif // ]
 #define FFSPRINTF(s,size,format,...) sprintf(s,format,__VA_ARGS__)
 #define FFSNPRINTF(s,size,format,...) snprintf(s,size,format,__VA_ARGS__)
 #define FFMESSAGE(m) PBU_DMESSAGE(m)
@@ -151,7 +159,7 @@ typedef char ffchar_t;
   #define FF_AVFILTER_V PBU_STR(LIBAVFILTER_VERSION_MAJOR)
 #endif // ]
 
-#if defined (_WIN32) // [
+#if defined (_WIN32) // [ /// WINDOWS ////////////////////////////////////////////
 #if defined (FF_AVUTIL_V) // [
 #define FF_AVUTIL L"avutil-" PBU_WIDEN(FF_AVUTIL_V) L".dll"
 #else // ] [
@@ -187,7 +195,43 @@ typedef char ffchar_t;
 #else // ] [
 #define FF_AVFILTER L"avfilter.dll"
 #endif // ]
+#elif defined __APPLE__ // ] [ /// APPLE //////////////////////////////////////////////
+#if defined (FF_AVUTIL_V) // [
+#define FF_AVUTIL "libavutil." FF_AVUTIL_V ".dylib"
 #else // ] [
+#define FF_AVUTIL "libavutil.dylib"
+#endif // ]
+#if defined (FF_AVCODEC_V) // [
+#define FF_AVCODEC "libavcodec." FF_AVCODEC_V  ".dylib"
+#else // ] [
+#define FF_AVCODEC "libavcodec.dylib"
+#endif // ]
+#if defined (FF_AVFORMAT_V) // [
+#define FF_AVFORMAT "libavformat." FF_AVFORMAT_V  ".dylib"
+#else // ] [
+#define FF_AVFORMAT "libavformat.dylib"
+#endif // ]
+#if defined (FF_SWRESAMPLE_V) // [
+#define FF_SWRESAMPLE "libswresample." FF_SWRESAMPLE_V  ".dylib"
+#else // ] [
+#define FF_SWRESAMPLE "libswresample.dylib"
+#endif // ]
+#if defined (FF_SWSCALE_V) // [
+#define FF_SWSCALE "libswscale." FF_SWSCALE_V  ".dylib"
+#else // ] [
+#define FF_SWSCALE "libswscale.dylib"
+#endif // ]
+#if defined (FF_POSTPROC_V) // [
+#define FF_POSTPROC "libpostproc." FF_POSTPROC_V  ".dylib"
+#else // ] [
+#define FF_POSTPROC "libpostproc.dylib"
+#endif // ]
+#if defined (FF_AVFILTER_V) // [
+#define FF_AVFILTER "libavfilter." FF_AVFILTER_V  ".dylib"
+#else // ] [
+#define FF_AVFILTER "libavfilter.dylib"
+#endif // ]
+#else // ] [ /// LINUX //////////////////////////////////////////////////////////////
 #if defined (FF_AVUTIL_V) // [
 #define FF_AVUTIL "libavutil.so." FF_AVUTIL_V
 #else // ] [
@@ -235,9 +279,11 @@ const char *ff_dynload_path(void);
 ///////////////////////////////////////////////////////////////////////////////
 #define FF_PACKET_UNREF
 #define FF_FRAME_UNREF
-// as it seems it is impossible to pass through a partial flac stream
-// (cf. "https://trac.ffmpeg.org/ticket/7864".) our hack consists in
-// reading/decoding and re-encoding/writing.
+/*
+ * as it seems it is impossible to pass-through a partial flac stream
+ * (cf. "https://trac.ffmpeg.org/ticket/7864".) our hack consists in
+ * reading/decoding and re-encoding/writing.
+ */
 #define FF_FLAC_HACK
 #define FF_STREAM_METADATA
 #define FF_RESAMPLER_RATE
@@ -245,28 +291,29 @@ const char *ff_dynload_path(void);
 
 #if defined (HAVE_FF_DYNLOAD) // [
 ///////////////////////////////////////////////////////////////////////////////
-// avfilter
-//   avcodec
-//     avutil
-//     swresample
-//   avformat
-//     avutil
-//     avcodec
-//   avutil
-//   postproc
-//     avutil
-//   swresample
-//     avutil
-//   swscale
-//     avutil
-//
-// avutil
-// swresample
-// avcodec
-// avformat
-// swscale
-// postproc
-// avfilter
+/*
+ * avfilter
+ *   avcodec
+ *     avutil
+ *     swresample
+ *   avformat
+ *     avutil
+ *     avcodec
+ *   avutil
+ *   postproc
+ *     avutil
+ *   swresample
+ *     avutil
+ *   swscale
+ *     avutil
+ * avutil
+ * swresample
+ * avcodec
+ * avformat
+ * swscale
+ * postproc
+ * avfilter
+ */
 #endif // ]
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -287,6 +334,7 @@ int ff_mv(const ffchar_t *source, const ffchar_t *target);
 int ff_mkdir(ffchar_t *path);
 
 ///////////////////////////////////////////////////////////////////////////////
+typedef enum ff_lfe ff_lfe_t;
 typedef const struct ff_iter_vmt ff_iter_vmt_t;
 typedef struct ff_iter ff_iter_t;
 typedef struct ff_param_decode ff_param_decode_t;
@@ -306,6 +354,21 @@ typedef enum ff_muxer_state ff_muxer_state_t;
 typedef struct ff_muxer ff_muxer_t;
 typedef struct ff_printer ff_printer_t;
 
+///////////////////////////////////////////////////////////////////////////////
+/*
+ * if there are more than 6 channels, the layout is not set by the decoder,
+ * and instead the default libavcodec channel layout guess is used
+ * -> 7 channels, 6.1 (front left, right, center, lfe, back center,
+ *    side left, right)
+ * -> 8 channels, 7.1 (front left, right, center, lfe, back left,
+ *    right, side left, right)
+ * http://lists.xiph.org/pipermail//flac-dev/2013-January/003569.html
+ */
+enum ff_lfe {
+  FF_LFE_THRESHOLD=6,
+  FF_LFE_CHANNEL=3,
+};
+
 ///////////////////////////////////////////////////////////////////////////////
 struct ff_iter_vmt {
   const char *id;
@@ -427,10 +490,21 @@ void ff_printer_clear(ff_printer_t *p);
 void ff_printer_reset(ff_printer_t *p);
 void ff_printer_flush(ff_printer_t *p);
 
+#if 0 // [
 int ff_printer_printf(ff_printer_t *p, const char *format, ...);
 #if defined (_WIN32) // [
 int ff_printer_wprintf(ff_printer_t *p, const wchar_t *format, ...);
 #endif // ]
+#else // ] [
+#if defined (WIN32) // [
+#define _FF_PRINTER_PRINTF(p,format,...) \
+	ff_printer_printf(p,L##format,__VA_ARGS__)
+#else // ] [
+#define _FF_PRINTER_PRINTF(p,format,...) \
+	ff_printer_printf(p,format,__VA_ARGS__)
+#endif // ]
+int ff_printer_printf(ff_printer_t *p, const ffchar_t *format, ...);
+#endif // ]
 
 struct ff_inout {
   struct {
