@@ -1,7 +1,7 @@
 /*
  * ff_dynload.c
  *
- * Copyright (C) 2014 Peter Belkner <pbelkner@users.sf.net>
+ * Copyright (C) 2014 Peter Belkner <info@pbelkner.de>
  * Nanos gigantum humeris insidentes #TeamWhite
  *
  * This program is free software; you can redistribute it and/or
@@ -25,11 +25,14 @@
 #if defined (_WIN32) // [
 #include <windows.h>
 #else // ] [
-#include <linux/limits.h>
+//#include <linux/limits.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <dlfcn.h>
 #endif // ]
+#if defined (__APPLE__) && defined (HAVE_LIBPROC) // [
+#include <libproc.h>
+#endif // ]
 
 ///////////////////////////////////////////////////////////////////////////////
 #if 1 // [
@@ -72,7 +75,7 @@ static void ff_dynload_list_append(ff_dynload_node_t *node,
   node->prev=ff_dynload_tail;
 #if defined (PBU_DEBUG) // [
   node->id=id;
-FFVWRITELN("\"%s\"",id);
+//FFVWRITELN("\"%s\"",id);
 #endif // ]
 
   ff_dynload_tail=node;
@@ -594,6 +597,8 @@ static struct _ff_avcodec {
   unsigned (*avcodec_version)(void);
 #if (LIBAVCODEC_VERSION_MAJOR<58) // [
   void (*avcodec_register_all)(void);
+	AVCodec *(*av_codec_next)(const AVCodec *c);
+	AVOutputFormat *(*av_oformat_next)(const AVOutputFormat *f);
 #endif // ]
   AVCodec *(*avcodec_find_decoder)(enum AVCodecID id);
   AVCodec *(*avcodec_find_decoder_by_name)(const char *name);
@@ -675,6 +680,22 @@ void avcodec_register_all(void)
 
   avcodec.avcodec_register_all();
 }
+
+AVCodec *av_codec_next(const AVCodec *c)
+{
+  if (avcodec_load_sym(&avcodec.av_codec_next,__func__)<0)
+    return NULL;
+
+  return avcodec.av_codec_next(c);
+}
+
+AVOutputFormat *av_oformat_next(const AVOutputFormat *f)
+{
+  if (avcodec_load_sym(&avcodec.av_oformat_next,__func__)<0)
+    return NULL;
+
+  return avcodec.av_oformat_next(f);
+}
 #endif // ]
 
 AVCodec *avcodec_find_decoder(enum AVCodecID id)
@@ -1288,7 +1309,11 @@ static struct _ff_avfilter {
   void (*avfilter_register_all)(void);
 #endif // ]
   AVFilterGraph *(*avfilter_graph_alloc)(void);
+#if (LIBAVFILTER_VERSION_MAJOR<7) // [
+  AVFilter *(*avfilter_get_by_name)(const char *name);
+#else // ] [
   const AVFilter *(*avfilter_get_by_name)(const char *name);
+#endif // ]
   int (*avfilter_graph_create_filter)(AVFilterContext **filt_ctx,
       const AVFilter *filt, const char *name, const char *args,
       void *opaque, AVFilterGraph *graph_ctx);
@@ -1333,7 +1358,11 @@ AVFilterGraph *avfilter_graph_alloc(void)
   return avfilter.avfilter_graph_alloc();
 }
 
+#if (LIBAVFILTER_VERSION_MAJOR<7) // [
+AVFilter *avfilter_get_by_name(const char *name)
+#else // ] [
 const AVFilter *avfilter_get_by_name(const char *name)
+#endif // ]
 {
   if (avfilter_load_sym(&avfilter.avfilter_get_by_name,__func__)<0)
     return NULL;
@@ -1435,7 +1464,7 @@ static int load(void *hLib, const char *sym, void *p)
     *fp=FFDLSYM(hLib,sym);
 
     if (!*fp) {
-      FFVMESSAGE("loading %s",sym);
+      _DMESSAGEV("loading %s",sym);
       return -1;
     }
   }
@@ -1464,7 +1493,11 @@ static int avutil_load(void)
 
   if (!avutil.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_AVUTIL)) {
-      FFVMESSAGE("loading %s",AVUTIL);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",AVUTIL);
+#else // ] [
+      _DMESSAGEV("loading %s",AVUTIL);
+#endif // ]
       return -1;
     }
 
@@ -1472,7 +1505,11 @@ static int avutil_load(void)
     avutil.node.hLib=FFDLOPEN(path,DLOPEN_FLAG);
 
     if (!avutil.node.hLib&&!(avutil.node.hLib=FFDLOPEN(AVUTIL,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1499,7 +1536,11 @@ static int avcodec_load(void)
     return -1;
   else if (!avcodec.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_AVCODEC)) {
-      FFVMESSAGE("loading %s",AVCODEC);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",AVCODEC);
+#else // ] [
+      _DMESSAGEV("loading %s",AVCODEC);
+#endif // ]
       return -1;
     }
 
@@ -1507,7 +1548,11 @@ static int avcodec_load(void)
     avcodec.node.hLib=FFDLOPEN(path,DLOPEN_FLAG);
 
     if (!avcodec.node.hLib&&!(avcodec.node.hLib=FFDLOPEN(AVCODEC,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1534,7 +1579,11 @@ static int avformat_load(void)
     return -1;
   else if (!avformat.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_AVFORMAT)) {
-      FFVMESSAGE("loading %s",AVFORMAT);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",AVFORMAT);
+#else // ] [
+      _DMESSAGEV("loading %s",AVFORMAT);
+#endif // ]
       return -1;
     }
 
@@ -1542,7 +1591,11 @@ static int avformat_load(void)
     avformat.node.hLib=FFDLOPEN(path,DLOPEN_FLAG);
 
     if (!avformat.node.hLib&&!(avformat.node.hLib=FFDLOPEN(AVFORMAT,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1567,7 +1620,11 @@ static int swresample_load(void)
     return -1;
   else if (!swresample.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_SWRESAMPLE)) {
-      FFVMESSAGE("loading %s",SWRESAMPLE);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",SWRESAMPLE);
+#else // ] [
+      _DMESSAGEV("loading %s",SWRESAMPLE);
+#endif // ]
       return -1;
    }
 
@@ -1576,7 +1633,11 @@ static int swresample_load(void)
 
     if (!swresample.node.hLib
         &&!(swresample.node.hLib=FFDLOPEN(SWRESAMPLE,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1601,7 +1662,11 @@ static int swscale_load(void)
     return -1;
   else if (!swscale.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_SWSCALE)) {
-      FFVMESSAGE("loading %s",SWSCALE);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",SWSCALE);
+#else // ] [
+      _DMESSAGEV("loading %s",SWSCALE);
+#endif // ]
       return -1;
    }
 
@@ -1609,7 +1674,11 @@ static int swscale_load(void)
     swscale.node.hLib=FFDLOPEN(path,DLOPEN_FLAG);
 
     if (!swscale.node.hLib&&!(swscale.node.hLib=FFDLOPEN(SWSCALE,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1634,7 +1703,11 @@ static int postproc_load(void)
     return -1;
   else if (!postproc.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_POSTPROC)) {
-      FFVMESSAGE("loading %s",POSTPROC);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",POSTPROC);
+#else // ] [
+      _DMESSAGEV("loading %s",POSTPROC);
+#endif // ]
       return -1;
     }
 
@@ -1642,7 +1715,11 @@ static int postproc_load(void)
     postproc.node.hLib=FFDLOPEN(path,DLOPEN_FLAG);
 
     if (!postproc.node.hLib&&!(postproc.node.hLib=FFDLOPEN(POSTPROC,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1677,7 +1754,11 @@ static int avfilter_load(void)
     return -1;
   else if (!avfilter.node.hLib) {
     if ((path+SIZE_PATH)<=(pp+SIZE_AVFILTER)) {
-      FFVMESSAGE("loading %s",AVFILTER);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",AVFILTER);
+#else // ] [
+      _DMESSAGEV("loading %s",AVFILTER);
+#endif // ]
       return -1;
     }
 
@@ -1685,7 +1766,11 @@ static int avfilter_load(void)
     avfilter.node.hLib=FFDLOPEN(path,DLOPEN_FLAG);
 
     if (!avfilter.node.hLib&&!(avfilter.node.hLib=FFDLOPEN(AVFILTER,DLOPEN_FLAG))) {
-      FFVMESSAGE("loading %s",path);
+#if defined (_WIN32) // [
+      _DMESSAGEV("loading %S",path);
+#else // ] [
+      _DMESSAGEV("loading %s",path);
+#endif // ]
       return -1;
     }
 
@@ -1795,10 +1880,11 @@ static int ff_dynload_relative(const ffchar_t *dirname)
   enum { SIZE=(sizeof path)/(sizeof path[0]) };
   int code=-1;
   ffchar_t *mp=path+SIZE;
-#if ! defined (_WIN32) // [
+#if defined (__linux__) // [
   char process_path[64];
 #endif // ]
-  int len,size;
+  int len;
+  int size;
 
   /////////////////////////////////////////////////////////////////////////////
 #if defined (_WIN32) // [
@@ -1806,6 +1892,11 @@ static int ff_dynload_relative(const ffchar_t *dirname)
 
 	if (ERROR_INSUFFICIENT_BUFFER==GetLastError())
     goto exit;
+#elif defined (__APPLE__) // ] [
+	proc_pidpath(getpid(),pp,path+(sizeof path)-pp);
+	len=strlen(pp);
+	//pp+=strlen(pp);
+_DWRITELNV("\"%s\"",path);
 #else // ] [
   sprintf(process_path,"/proc/%d/exe",getpid());
 
@@ -1842,6 +1933,9 @@ static int ff_dynload_relative(const ffchar_t *dirname)
   *pp=FFL('\0');
 
   /////////////////////////////////////////////////////////////////////////////
+#if defined (__APPLE__) // [
+_DWRITELNV("\"%s\"",path);
+#endif // ]
   code=0;
 exit:
   return code;
@@ -1890,9 +1984,9 @@ int ff_dynload(const ffchar_t *dirname)
 
   if (code<0) {
 #if defined (_WIN32) // [
-    DVMESSAGEW(L"setting dirname \"%s\"",dirname);
+    _DMESSAGEV("setting dirname \"%S\"",dirname);
 #else // ] [
-    DVMESSAGE("setting dirname \"%s\"",dirname);
+    _DMESSAGEV("setting dirname \"%s\"",dirname);
 #endif // ]
     goto exit;
   }
@@ -1908,7 +2002,7 @@ void ff_unload(void)
 {
 #if defined (FF_DYNLOAD_FREE_LIST) // [
   while (ff_dynload_tail) {
-FFVWRITELN("\"%s\"",ff_dynload_tail->id);
+//FFVWRITELN("\"%s\"",ff_dynload_tail->id);
 #if defined (_WIN32) // [
     FreeLibrary(ff_dynload_tail->hLib);
 #else // ] [
